; Especificaciones: REPLICANTE para ZX Spectrum 16K/48K.
; Descripción general: Utilizando las rutinas LOAD y SAVE de la
; memoria ROM se realizan réplicas exactas de cualquier programa.
; Longitud: 232 bytes + 14 bytes de Move.
; Entrada: Ningún requerimiento.
; Salida: Ningún requerimiento.
; Registros usados: A, B, C, D, E, H, L e IX en programa REPLICANTE.
;					A, B, C, D, E, H, L e IX en rutinas ROM
org		$7000

b_printer	equ $5b00
cabecera	equ $5bed
cls			equ $0d6b
ld_bytes	equ $0556
long		equ $5beb
open		equ $1601
pr_string	equ $203c
replica		equ $5dad
sa_bytes 	equ $04c2
sa_control	equ $0970

; ORG $5B00 da error de RAMTOP
; Se mueve al buffer de la impresora y se ejecuta desde allí, dirección 23296
Move:
ld		hl, Main						; HL = Inicio del programa
ld		de, b_printer					; DE = Buffer de impresora
ld		bc, EndProg-Main				; BC = Longitud del programa
ldir									; Pasa el programa al buffer de impresora
jp		b_printer						; Ejecuta el programa en el buffer de impresora

Main:
call	cls								; Borra pantalla
ld		a, 2							; Abre
call	open							; canal número dos
ld		de, Mens						; Dirección mensaje
ld		bc, 115							; Longitud mensaje
call	pr_string						; Imprímelo
Menu:
ld		a, 54							; Semifila Cs-V
in		a, ($fe)						; Lee semifila
bit		3, a							; ¿C pulsada?
jr		z, Load							; Si pulsada, Load
bit		2, a							; ¿X pulsada?
jr		z, LoadData						; Si pulsada, LoadData
ld		a, 253							; Semifila A-G
in		a, ($fe)						; Lee semifila
bit		1, a							; ¿S pulsada?
jr		z, Save							; Si pulsada, Save
bit		0, a							; ¿A pulsada?
jr		z, SaveData						; Si pulsada, SaveData
ld		a, 251							; Semifila Q-T
in		a, ($fe)						; Lee semifila
bit		3, a							; ¿R pulsada?
jr		nz, Menu						; Si no pulsada, Menu
ret										; Vuelve al Basic

Load:
scf										; Carry = 1, LOAD
ld		a, $00							; Señal de cabecera
ld		ix, cabecera					; IX = Base Address, direccion donde se carga la cabecera
ld		de, $11							; Número de bytes a cargar, 17
call	ld_bytes						; Carga la cabecera
LoadData:
scf										; Carry = 1, LOAD
ld		a, $ff							; Señal de datos
ld		ix, replica						; IX = Base Address, dirección donde se cargan los datos
ld		de, $ffff						; Número de bytes a carga, 65535 (el máximo)
call	ld_bytes						; Carga los datos
push	ix
pop		hl								; HL = IX, dirección del último byte cargado
ld		de, replica						; DE = Base Address, dirección donde se cargan los datos
xor		a								; Carry = 0
sbc		hl, de							; HL = HL - DE, HL = número de bytes cargados
ld		(long), hl						; Almacena el resultado
LoadDataEnd:
jr		Menu							; Regresa al menú
Save:
ld		a, $00							; Señal de cabecera
ld		hl, replica+$01					; HL = Dirección desde la que se graban los datos
ld		ix, cabecera					; IX = Base Address, direccion donde se carga la cabecera
call	sa_control						; Graba la cabecera
SaveData:
ld		a, $ff							; Señal de datos
ld		ix, replica+$01					; IX = Dirección desde la que se graban los datos
ld		de, (long)						; DE = longitud de los datos a grabar
call	sa_bytes						; Graba los datos
SaveDataEnd:
jr		LoadDataEnd

Mens:
db		$16, $00, $0a, $14, $01			; AT 0, 10; INVERSE 1
defm	"REPLICANTE"
db		$16, $03, $05					; ART 3, 4
defm	"C"
db		$14, $00						; INVERSE 0
defm	" LOAD"
db		$16, $05, $05, $14, $01			; AT 5, 5; INVERSE 1
defm	"X"
db		$14, $00						; INVERSE 0
defm	" LOAD Data"
db		$16, $07, $05, $14, $01			; AT 7, 5; INVERSE 1
defm	"S"
db		$14, $00						; INVERSE 0
defm	" SAVE"
db		$16, $09, $05, $14, $01			; AT 9, 5; INVERSE 1
defm	"A"
db		$14, $00						; INVERSE 0
defm	" SAVE Data"
db		$16, $0b, $05, $14, $01			; AT 11, 5; INVERSE 1
defm	"R"
db		$14, $00						; INVERSE 0
defm	" RET"
db		$16, $0f, $02					; AT 15, 2
defm	"Por Juan Mtz. Velarde 1984"
EndProg:

end		$7000